@@ -1,4 +1,8 @@
 
+`%||%` <- function(x, y) {
+  if (is.environment(x) || length(x)) x else y
+}
+
 bootstrap <- function(version, library) {
 
   # attempt to download renv
@@ -24,6 +28,11 @@ renv_bootstrap_repos <- function() {
   if (!is.na(repos))
     return(repos)
 
+  # check for lockfile repositories
+  repos <- tryCatch(renv_bootstrap_repos_lockfile(), error = identity)
+  if (!inherits(repos, "error") && length(repos))
+    return(repos)
+
   # if we're testing, re-use the test repositories
   if (renv_bootstrap_tests_running())
     return(getOption("renv.tests.repos"))
@@ -48,6 +57,30 @@ renv_bootstrap_repos <- function() {
 
 }
 
+renv_bootstrap_repos_lockfile <- function() {
+
+  lockpath <- Sys.getenv("RENV_PATHS_LOCKFILE", unset = "renv.lock")
+  if (!file.exists(lockpath))
+    return(NULL)
+
+  lockfile <- tryCatch(renv_json_read(lockpath), error = identity)
+  if (inherits(lockfile, "error")) {
+    warning(lockfile)
+    return(NULL)
+  }
+
+  repos <- lockfile$R$Repositories
+  if (length(repos) == 0)
+    return(NULL)
+
+  keys <- vapply(repos, `[[`, "Name", FUN.VALUE = character(1))
+  vals <- vapply(repos, `[[`, "URL", FUN.VALUE = character(1))
+  names(vals) <- keys
+
+  return(vals)
+
+}
+
 renv_bootstrap_download <- function(version) {
 
   # if the renv version number has 4 components, assume it must
@@ -254,7 +287,7 @@ renv_bootstrap_install <- function(version, tarball, library) {
   bin <- R.home("bin")
   exe <- if (Sys.info()[["sysname"]] == "Windows") "R.exe" else "R"
   r <- file.path(bin, exe)
-  args <- c("--vanilla", "CMD", "INSTALL", "-l", shQuote(library), shQuote(tarball))
+  args <- c("--vanilla", "CMD", "INSTALL", "--no-multiarch", "-l", shQuote(library), shQuote(tarball))
   output <- system2(r, args, stdout = TRUE, stderr = TRUE)
   message("Done!")
 
@@ -447,18 +480,33 @@ renv_bootstrap_library_root_name <- function(project) {
 
 renv_bootstrap_library_root <- function(project) {
 
+  prefix <- renv_bootstrap_profile_prefix()
+
   path <- Sys.getenv("RENV_PATHS_LIBRARY", unset = NA)
   if (!is.na(path))
-    return(path)
+    return(paste(c(path, prefix), collapse = "/"))
 
-  path <- Sys.getenv("RENV_PATHS_LIBRARY_ROOT", unset = NA)
-  if (!is.na(path)) {
+  path <- renv_bootstrap_library_root_impl(project)
+  if (!is.null(path)) {
     name <- renv_bootstrap_library_root_name(project)
-    return(file.path(path, name))
+    return(paste(c(path, prefix, name), collapse = "/"))
   }
 
-  prefix <- renv_bootstrap_profile_prefix()
-  paste(c(project, prefix, "renv/library"), collapse = "/")
+  renv_bootstrap_paths_renv("library", project = project)
+
+}
+
+renv_bootstrap_library_root_impl <- function(project) {
+
+  root <- Sys.getenv("RENV_PATHS_LIBRARY_ROOT", unset = NA)
+  if (!is.na(root))
+    return(root)
+
+  type <- renv_bootstrap_project_type(project)
+  if (identical(type, "package")) {
+    userdir <- renv_bootstrap_user_dir()
+    return(file.path(userdir, "library"))
+  }
 
 }
 
@@ -524,7 +572,7 @@ renv_bootstrap_profile_load <- function(project) {
     return(profile)
 
   # check for a profile file (nothing to do if it doesn't exist)
-  path <- file.path(project, "renv/local/profile")
+  path <- renv_bootstrap_paths_renv("profile", profile = FALSE)
   if (!file.exists(path))
     return(NULL)
 
@@ -535,7 +583,7 @@ renv_bootstrap_profile_load <- function(project) {
 
   # set RENV_PROFILE
   profile <- contents[[1L]]
-  if (nzchar(profile))
+  if (!profile %in% c("", "default"))
     Sys.setenv(RENV_PROFILE = profile)
 
   profile
@@ -545,7 +593,7 @@ renv_bootstrap_profile_load <- function(project) {
 renv_bootstrap_profile_prefix <- function() {
   profile <- renv_bootstrap_profile_get()
   if (!is.null(profile))
-    return(file.path("renv/profiles", profile))
+    return(file.path("profiles", profile, "renv"))
 }
 
 renv_bootstrap_profile_get <- function() {
@@ -569,3 +617,78 @@ renv_bootstrap_profile_normalize <- function(profile) {
   profile
 
 }
+
+renv_bootstrap_path_absolute <- function(path) {
+
+  substr(path, 1L, 1L) %in% c("~", "/", "\\") || (
+    substr(path, 1L, 1L) %in% c(letters, LETTERS) &&
+    substr(path, 2L, 3L) %in% c(":/", ":\\")
+  )
+
+}
+
+renv_bootstrap_paths_renv <- function(..., profile = TRUE, project = NULL) {
+  renv <- Sys.getenv("RENV_PATHS_RENV", unset = "renv")
+  root <- if (renv_bootstrap_path_absolute(renv)) NULL else project
+  prefix <- if (profile) renv_bootstrap_profile_prefix()
+  components <- c(root, renv, prefix, ...)
+  paste(components, collapse = "/")
+}
+
+renv_bootstrap_project_type <- function(path) {
+
+  descpath <- file.path(path, "DESCRIPTION")
+  if (!file.exists(descpath))
+    return("unknown")
+
+  desc <- tryCatch(
+    read.dcf(descpath, all = TRUE),
+    error = identity
+  )
+
+  if (inherits(desc, "error"))
+    return("unknown")
+
+  type <- desc$Type
+  if (!is.null(type))
+    return(tolower(type))
+
+  package <- desc$Package
+  if (!is.null(package))
+    return("package")
+
+  "unknown"
+
+}
+
+renv_bootstrap_user_dir <- function(path) {
+  dir <- renv_bootstrap_user_dir_impl(path)
+  chartr("\\", "/", dir)
+}
+
+renv_bootstrap_user_dir_impl <- function(path) {
+
+  # use R_user_dir if available
+  tools <- asNamespace("tools")
+  if (is.function(tools$R_user_dir))
+    return(tools$R_user_dir("renv", "cache"))
+
+  # try using our own backfill for older versions of R
+  envvars <- c("R_USER_CACHE_DIR", "XDG_CACHE_HOME")
+  for (envvar in envvars) {
+    root <- Sys.getenv(envvar, unset = NA)
+    if (!is.na(root)) {
+      path <- file.path(root, "R/renv")
+      return(path)
+    }
+  }
+
+  # use platform-specific default fallbacks
+  if (Sys.info()[["sysname"]] == "Windows")
+    file.path(Sys.getenv("LOCALAPPDATA"), "R/cache/R/renv")
+  else if (Sys.info()[["sysname"]] == "Darwin")
+    "~/Library/Caches/org.R-project.R/R/renv"
+  else
+    "~/.cache/R/renv"
+
+}
