@@ -30,31 +30,33 @@
 #' @export
 consent <- function(provided = FALSE) {
 
-  renv_scope_options(renv.consenting = TRUE)
-
+  # compute path to root directory
   root <- renv_paths_root()
   if (renv_file_type(root) == "directory") {
     vwritef("* Consent to use renv has already been provided -- nothing to do.")
     return(invisible(TRUE))
   }
 
+  # write welcome message
   template <- system.file("resources/WELCOME", package = "renv")
   contents <- readLines(template)
   replacements <- list(RENV_PATHS_ROOT = renv_path_pretty(root))
   welcome <- renv_template_replace(contents, replacements)
   writeLines(welcome)
 
+  # ask user if they want to proceed
   response <- catchall(proceed(default = provided))
   if (!identical(response, TRUE)) {
     msg <- "consent was not provided; operation aborted"
     stop(msg, call. = FALSE)
   }
 
+  # cache the user response
   options(renv.consent = TRUE)
   ensure_directory(root)
   vwritef("* %s has been created.", renv_path_pretty(root))
 
-  return(invisible(TRUE))
+  invisible(TRUE)
 
 }
 
@@ -68,7 +70,6 @@ renv_consent_check <- function() {
     stopf("consent has been explicitly withdrawn")
 
   # check for existence of root
-  renv_scope_options(renv.consenting = TRUE)
   root <- renv_paths_root()
   if (renv_file_type(root) == "directory")
     return(TRUE)
@@ -79,16 +80,15 @@ renv_consent_check <- function() {
     !is.na(Sys.getenv("CI", unset = NA)) ||
     !is.na(Sys.getenv("GITHUB_ACTION", unset = NA)) ||
     !is.na(Sys.getenv("RENV_PATHS_ROOT", unset = NA)) ||
+    file.exists("/.singularity.d") ||
     file.exists("/.dockerenv")
 
-  if (consented)
+  if (consented) {
+    ensure_directory(root)
     return(TRUE)
-
-  if (!interactive()) {
-    msg <- "please call `renv::consent()` before using renv"
-    stop(msg, call. = FALSE)
   }
 
+  # looks like the user's first interactive use of renv
   consent()
 
 }
