@@ -1,18 +1,18 @@
 
-renv_request_restart <- function(project, reason, ...) {
+renv_restart_request <- function(project, reason, ...) {
 
   # if we're running in RStudio, explicitly open the project
   # if it differs from the current project
   if (renv_rstudio_available()) {
-    status <- renv_request_restart_rstudio(project, reason, ...)
+    status <- renv_restart_request_rstudio(project, reason, ...)
     return(invisible(status))
   }
 
-  renv_request_restart_default(project, reason, ...)
+  renv_restart_request_default(project, reason, ...)
 
 }
 
-renv_request_restart_default <- function(project, reason, ...) {
+renv_restart_request_default <- function(project, reason, ...) {
 
   # use 'restart' helper defined by front-end (if any)
   restart <- getOption("restart")
@@ -27,16 +27,16 @@ renv_request_restart_default <- function(project, reason, ...) {
 
 }
 
-renv_request_restart_rstudio <- function(project, reason, ...) {
+renv_restart_request_rstudio <- function(project, reason, ...) {
 
   # if we're running tests, don't restart
   if (renv_tests_running())
-    return(renv_request_restart_default(project, reason, ...))
+    return(renv_restart_request_default(project, reason, ...))
 
   # if we don't have a tools env, bail
   tools <- catch(as.environment("tools:rstudio"))
   if (inherits(tools, "error"))
-    return(renv_request_restart_default(project, reason, ...))
+    return(renv_restart_request_default(project, reason, ...))
 
   # if RStudio is too old, use default restart impl
   old <-
@@ -44,7 +44,7 @@ renv_request_restart_rstudio <- function(project, reason, ...) {
     is.null(tools$.rs.api.openProject)
 
   if (old)
-    return(renv_request_restart_default(project, reason, ...))
+    return(renv_restart_request_default(project, reason, ...))
 
   # if the requested project matches the current project, just
   # restart the R session
