@@ -7,11 +7,6 @@ local({
   # the project directory
   project <- getwd()
 
-  # figure out path to 'renv' folder from this script
-  call <- sys.call(1L)
-  if (is.call(call) && identical(call[[1L]], as.symbol("source")))
-    Sys.setenv(RENV_PATHS_RENV = dirname(call[[2L]]))
-
   # figure out whether the autoloader is enabled
   enabled <- local({
 
